'use strict';


module.exports = function(def) {

  const init = def.cli
    .command('init', '初始化工程目录结构')
    .alias('it')
    .group('kit')
    .action(function* () {
      const showedKits = def.kits
        .filter(k => k.showed)
        .map(k => {
          return {
            'name': pad(k.name, 5) + '  ' + k.desc,
            'value': k.type,
            'short': k.name
          }
        });

      let type = yield def.ui.list('请选择套件：', showedKits);
      yield def.useKit(type);
      def.log.verbose('core', 'use kit `%s` in init', type);

      const parsed = yield def.cli.parse(process.argv.slice(2), {'use': 'parser'});
      const data = def.cli._apply(parsed);

      yield def.runKit('init', [type], data.opts, data.ctx, data.args);
    });

  const add = def.cli
    .command('add [type] [names...]', '创建工程脚手架')
    .alias('a')
    .argument('[type]', '类型，由具体套件提供')
    .argument('[names...]', '名称，通过空格分隔指定多个')
    .group('kit')
    .action(function* (type, names, opts) {
      let args = [];
      if (names != null) {
        args = args.concat(names);
      }

      let choices = add._args[0].choices;
      if (type == null && choices) {
        let list = [];
        let max = Object.keys(choices).reduce((max, c) => Math.max(max, c.length), 0);
        for (let k in choices) {
          list.push({
            'name': pad(k, max) + '  ' + choices[k],
            'value': k,
            'short': k
          });
        }
        let t = yield def.ui.list('请选择类型：', list);
        if (t == '*') {
          t = '';
        }
        args.unshift(t);
        arguments[0] = t;
      } else {
        args.unshift(type);
      }

      yield def.runKit('add', args, opts, this, arguments);
    });

  const dev = def.cli
    .command('dev', '开启本地调试服务器')
    .alias('d')
    .group('kit')
    .action(function* (opts) {
      def.serve.cmdOpts = opts;
      yield def.runKit('dev', [], opts, this, arguments);
    });

  const build = def.cli
    .command('build', '执行代码构建')
    .alias('b')
    .group('kit')
    .action(function* (opts) {
      def.builder.cmdOpts = opts;
      yield def.runKit('build', [], opts, this, arguments);
    });

  const test = def.cli
    .command('test [type]', '执行代码测试')
    .alias('t')
    .group('kit')
    .action(function* (type, opts) {
      let args = [];
      if (type != null) {
        args.push(type);
      }

      yield def.runKit('test', args, opts, this, arguments);
    });

  const publish = def.cli
    .command('publish', '执行代码发布')
    .alias('p')
    .group('kit')
    .action(function* (opts) {
      yield def.runKit('publish', [], opts, this, ['', opts]);
    });

  return {
    'init': init,
    'add': add,
    'build': build,
    'dev': dev,
    'test': test,
    'publish': publish
  };

  function pad(str, n) {
    str = str.split('');
    n = n - str.length;
    if (n > 0) {
      for (var i = 0; i < n; i += 1) {
        str.push(' ');
      }
    }
    str.push(' :');
    return str.join('');
  }

};