'use strict';

module.exports = function(ctx) {

  const spec = {
    'init': {
      'action': function* (opts) {
        ctx.log.info('[core]', '默认套件无此功能，请尝试使用 DEF 官方套件.');
      }
    },
    'add': {
      'action': function* (opts) {
        ctx.log.info('[core]', '默认套件无此功能，请尝试使用 DEF 官方套件.');
      }
    },
    'dev': {
      'options': {
        'port': {
          'alias': 'p',
          'description': '调试服务器监听端口',
          'default': 3333
        },
        'lrPort': {
          'alias': 'r',
          'description': 'live reload 服务器监听端口',
          'type': 'number'
        },
        'open-url': {
          'alias': 'o',
          'description': '自动打开预览 url',
          'default': false
        },
        'mobile-preview': {
          'alias': 'm',
          'description': '自动打开无线预览 url',
          'default': false
        },
        'no-livereload': {
          'alias': 'L',
          'description': '禁止 live reload'
        }
      },
      'action': function* (opts) {
        yield ctx.serve.start(opts);
      }
    },
    'build': {
      'options': {
        'local': {
          'alias': 'l',
          'description': '使用本地安装的 builder 执行构建'
        },
        'buildTo': {
          'alias': 't',
          'description': '构建代码存放目录',
          'default': 'build'
        },
        'debug': {
          'alias': 'd',
          'description': '生成构建后未压缩代码',
          'default': false
        },
        'type': {
          'description': '指定构建类型，默认为 assets',
          'default': 'assets'
        },
        'env': {
          'alias': 'e',
          'description': '预留调试参数，请勿使用',
          'default': 'production',
          'hide': true
        },
        'api': {
          'description': '预留调试参数，请勿使用',
          'default': null,
          'hide': true
        }
      },
      'action': function* (opts) {
        yield ctx.builder.run({
          'buildTo': opts.buildTo,
          'env': opts.env,
          'cloudBuild': !opts.local,
          'argv': opts['--'],
          'debug': opts.debug,
          'type': opts.type,
          'api': opts.api
        });
      }
    },
    'test': {
      'action': function* (type, opts) {
        ctx.log.info('[core]', '默认套件无此功能，请尝试使用 DEF 官方套件');
      }
    },
    'publish': {
      'action': function* (type, opts) {
        yield ctx.deploy.run(type, opts);
      }
    }
  };

  const registerPublishCmd = function(command) {
    command
      .option('-e, --env [env]', '发布环境', {'hide': true})
      .option('-d, --daily', '发布到日常环境')
      .option('-u, --prepub', '发布到预发环境（仅对 webapp、package 生效）')
      .option('-r, --preprod', '发布到线上预览环境（仅对 webapp、package 生效）')
      .option('-o, --prod', '发布到线上环境')
        .command('assets', '发布 assets（JS、CSS）')
        .option('--push', '发布前强制 push 代码')
        .option('--clear', '线上发布成功后切换到 master 分支并同步远程代码')
        .option('-D, --no-deletebranch', '线上发布成功后不删除源 daily 分支')
        .action(function* (opts) {
          yield ctx.runKit('publish', [], opts, this, ['assets', opts]);
        })
        .end()
        .command('webapp', '发布 webapp html 页面')
        .option('-c, --config', '配置 webapp 或者 package')
        .action(function* (opts) {
          yield ctx.runKit('publish', [], opts, this, ['webapp', opts]);
        })
        .end()
        .command('package', '发布 zip 包')
        .option('-c, --config', '配置 webapp 或者 package')
        .option('--field <field>', '指定发布的离线包，用于多离线包发布（兼容旧版）')
        .option('-U, --no-upload', '只生成离线包，不执行发布')
        .action(function* (opts) {
          yield ctx.runKit('publish', [], opts, this, ['package', opts]);
        })
        .end();
  };

  return {
    'spec': spec,
    'registerPublishCmd': registerPublishCmd
  };

};