"use strict";

var path  =require('path');
var fs  = require('fs');
var minimatch = require('minimatch');
var defaults = require('lodash.defaults');

module.exports = function(src, dest, options) {

  var log = console.log;

  options = defaults(options || {}, {
    compile: function(srcFile, destFile, cb) {
      cb(new Error('To be implemented'));
    },
    resolvePath: null
  });

  if (!options.patten) {
    options.patten = [];
  } else if (!Array.isArray(options.patten)) {
    options.patten = [options.patten];
  }

  var patten = options.patten.filter(function(patten) {
    return patten[0] !== '!';
  });

  var negativePatten = options.patten.filter(function(patten) {
    return patten[0] === '!';
  }).map(function(patten) {
    return patten.substr(1);
  });

  function isPathValid(pathname) {
    var matchPatten = patten.length > 0 && patten.some(function(patten) {
      return minimatch(pathname, patten);
    });

    var matchNegative = negativePatten.length > 0 && negativePatten.some(function(patten) {
      return minimatch(pathname, patten);
    });

    return matchPatten && !matchNegative;
  }

  return function(req, res, next) {
    if ('GET' != req.method.toUpperCase() && 'HEAD' != req.method.toUpperCase()) {
      return next();
    }

    var error = function(err) {
      return next('ENOENT' == err.code ? null : err);
    };

    var filePath = req.path;

    if (!isPathValid(filePath)) {
      return next();
    }

    var srcPath  = path.join(src, options.resolvePath ? options.resolvePath(filePath) : filePath);
    var destPath = path.join(dest, filePath);

    var compile = function() {
      options.compile(srcPath, destPath, function(err) {
        if (err) {
          error(err);
          return;
        }
        next()
      });
    };

    fs.stat(srcPath, function(err, srcStat) {
      if (err) {
        return error(err);
      }

      fs.stat(destPath, function(err, buildStat) {
        if (err) {
          if ('ENOENT' == err.code) {
            log('not found', destPath);
            return compile();
          } else {
            return next(err);
          }
        } else if (srcStat.mtime > buildStat.mtime) {
          // Source has changed, compile it
          log('modified', destPath);
          return compile();
        } else {
          return next();
        }
      });
    });

  }
};