'use strict';

var ejs = require('ejs');
var path = require('path');
var fs = require('fs');
var defaults = require('lodash.defaults');

module.exports = function(options) {
  var root = options.src || '.';
  var tplOptions = options.options || {};

  function isValidPath(pathname) {
    return path.extname(pathname) === '.ejs';
  }

  return function(req, res, next) {
    var pathname = req.path;

    if (!isValidPath(pathname)) {
      return next();
    }

    if ('no-render' in req.query) {
      return next();
    }

    var srcPath = path.join(root, pathname);
    var dataPath = srcPath.replace(/\.ejs$/, '.json');

    var error = function(err) {
      return next('ENOENT' == err.code ? null : err);
    };

    var render = function(tpl, data) {
      res.type('.html');
      try {
        var html = ejs.render(tpl, defaults(data || {} , {
          filename: srcPath,
          query: req.query
        }, tplOptions));
        res.end(html);
      } catch(e) {
        next(e);
        return;
      }
    };

    fs.readFile(srcPath, 'utf8', function(err, content) {
      if (err) {
        return error(err);
      }
      fs.exists(dataPath, function(exists) {
        if (exists) {
          fs.readFile(dataPath, function(err, strJSON) {
            if (err) {
              error(err);
              return;
            }

            try {
              var data = JSON.parse(strJSON);
            } catch(e) {
              error(e);
              return;
            }

            render(content, data);
          })
        } else {
          render(content);
        }
      });
    });

  };
};