'use strict';


const argv = require('yargs-parser')(process.argv);

const app = require(argv.app);
const reflect = require(argv.reflect);
const opts = Object.assign(
  JSON.parse(argv.opts), 
  {
    'notShowOpenTip': true, 
    'openUrl': false,
    'mobilePreview': false
  }
);

let express = null;


process.on('uncaughtException', error);
process.on('unhandledRejection', error);


process.on('message', function(message) {
  if (message.__from == 'main') {
    // restart
    if (message.__action == 'restart') {
      stop();
      start();
    }

    // start
    if (message.__action == 'start') {
      start();
    }

    // stop
    if (message.__action == 'stop') {
      stop();
    }
  }
});

function start() {
  if (!express) {
    app
      .start(reflect, opts)
      .then(s => {
        express = s;
        process.send({'__action': 'start'});
      })
      .catch(err => error(err));
  }
}

function stop() {
  if (express) {
    express.server.close();
    express = null;

    process.send({'__action': 'stop'});
  }
}

function error(err) {
  console.log(err.stack);
  
  process.send({
    '__error': {
      'message': err.message,
      'stack': err.stack
    }
  });
}
