'use strict';

const debug = require('debug')('def:runtime');
const Util = require('./util');

let ctx = null;

module.exports = function(def) {

  ctx = def;

  return {'create': create};

  function create(opts) {
    opts = opts || {};
    debug('create opts=', opts);

    const runtime = Object.create(null);

    // FIXME: 兼容用
    runtime.__fallback = true;

    baseRuntime(runtime, opts);
    if (opts.type == 'kit') {
      kitRuntime(runtime, opts);
    } else if (opts.type == 'plugin') {
      pluginRuntime(runtime, opts);
    }

    return runtime;
  }

  function baseRuntime(runtime, opts) {
    // env
    runtime.env = {
      'home': ctx._home,
      'version': ctx._cliVersion + '_' + ctx._version,
      'server': ctx._server,
      'next': true
    };

    // log
    runtime.log = ctx.log.Adapter(opts.type + ':' + opts.key);

    // config
    runtime.config = (function(name, type, config) {
      function key(k) {
        return type + ':' + name + ':' + k;
      }

      return {
        'get': function(k) {
          return config.get(key(k));
        },
        'set': function(k, v) {
          return config.set(key(k), v);
        },
        'save': function() {
          return config.save();
        }
      };
    })(opts.key, opts.type, ctx.config);

    // fatal
    runtime.fatal = ctx.fatal.bind(ctx);

    //
    Object.defineProperty(runtime, 'ui', {
      'get': function() { return ctx.ui; }
    });

    Object.defineProperty(runtime, 'io', {
      'get': function() { return ctx.io; }
    });

    Object.defineProperty(runtime, 'auth', {
      'get': function() { return ctx.auth; }
    });

    Object.defineProperty(runtime, 'repo', {
      'get': function() { return ctx.repo; }
    });

    //
    Object.defineProperty(runtime, 'Promise', {
      'get': function() { return ctx.Promise; }
    });

    //
    Object.defineProperty(runtime, 'util', {
      'get': function() { return ctx.util; }
    });

    //
    Object.defineProperty(runtime, 'parser', {
      'get': function() { return ctx.parser; }
    });

    // store
    runtime.store = {};
    Object.defineProperty(runtime.store, 'resolve', {
      'get': function() { return ctx.store.resolve.bind(ctx.store); }
    });
    Object.defineProperty(runtime.store, 'require', {
      'get': function() { return ctx.store.require.bind(ctx.store); }
    });
    // depreted
    Object.defineProperty(runtime.store, 'install', {
      'get': function() {
        return function* (names) {
          if (!Array.isArray(names)) {
            names = [names];
          }
          for (let i = 0, len = names.length; i < len; i += 1) {
            let name = names[i];
            if (Util.isBuilder(name)) {
              yield ctx.checkUpdate(name, 'builder');
            } else if (Util.isGenerator(name)) {
              yield ctx.checkUpdate(name, 'generator');
            }
          }
        };
      }
    });
  }

  function kitRuntime(runtime, opts) {
    const kit = opts.kit;

    // depreted
    runtime.login = {
      'getUserInfo': ctx.auth.user.bind(ctx.auth),
      'getUserTicket': ctx.auth.ticket.bind(ctx.auth)
    };

    runtime.kit = {};

    //yo
    runtime.kit.yo = {
      'run': function(opts) {
        const path = require('path');
        const fs = require('fs');
        const shorten = function(str) {
          return str.replace(
            new RegExp(path.join(ctx._home, 'def_modules', kit.name, 'node_modules', '.\\d+.\\d+.\\d+@'), 'ig'), 
            ''
          );
        };

        opts = opts || {};

        if (opts.resolved) {
          ctx.log.info('[core]', 'generator: %s [from kit]', shorten(opts.resolved));
          return ctx.yo.run(opts);
        }

        let spec = (opts.generator || opts.name).split(':');
        let name = spec[0].replace(/^(generator-)?(.+)$/, 'generator-$2');
        let sub = spec[1] || 'app';

        let kitRoot = ctx.store.resolve(kit.name);
        let resolved = path.join(kitRoot, 'node_modules', name, sub, 'index.js');
        if (!fs.existsSync(resolved)) {
          resolved = path.join(kitRoot, 'node_modules', name, 'generators', sub, 'index.js');
          if (!fs.existsSync(resolved)) {
            resolved = '';
          }
        }
        if (resolved) {
          opts.resolved = resolved;
          let pkg = null;
          try {
            pkg = fs.readFileSync(path.join(kitRoot, 'node_modules', name, 'package.json'), 'utf8');
            pkg = JSON.parse(pkg);
          } catch(e) {}

          if (pkg) {
            ctx.log.info('[core]', 'generator: %s@%s [from kit]', pkg.name, pkg.version);
          } else {
            ctx.log.info('[core]', 'generator: %s [from kit]', opts.resolved);
          }
        }
        return ctx.yo.run(opts);
      }
    };

    // depreted
    Object.defineProperty(runtime.kit, 'scm', {
      'get': function() { return ctx.repo; }
    });

    Object.defineProperty(runtime.kit, 'build', {
      'get': function() { return ctx.builder; }
    });
    Object.defineProperty(runtime.kit, 'reflect', {
      'get': function() { return ctx.serve; }
    });

    // look up abc.json
    runtime.lookupABCJson = runtime.lookupConfigJson = ctx.lookupConfigJson.bind(ctx);

    return runtime;
  }

  function pluginRuntime(runtime, opts) {
    const plugin = opts.plugin;

    const cli = Object.create(null);
    const cmd = plugin.cmd;

    cli.__proto__ = {};
    cli.option = function(flags, description, config) {
      return cmd.option(flags, description, config);
    };
    cli.usage = function(usage) {
      return cmd.usage(usage);
    };
    cli.argument = function(spec, description, choices) {
      return cmd.argument(spec, description, choices);
    };
    cli.description = function(desc) {
      return cmd.description(desc);
    };
    cli.action = function(fn) {
      return cmd.action(fn);
    };
    cli.cancel = function(fn) {
      return cmd.cancel(fn);
    };
    cli.command = function(spec, description) {
      return cmd.command(spec, description);
    };
    cli.outputHelp = function() {
      return cmd.showHelp();
    };
    cli.allowUnknownOption = function() {
      return cmd.allowUnknownOption();
    };

    runtime.cli = cli;

    // depreted
    runtime.request = function(action, args) {
      const co = require('co');

      const p = action.split(':');
      const name = p[0] || '';
      const method = p[1] || '';

      ctx.log.verbose('[core]', 'p `%s` request `%s`', plugin.key, action, args);

      let obj = {}, usePlugin = false;
      switch (name) {
        case 'build':
          obj = ctx.builder;
          break;
        case 'yo':
          obj = ctx.yo;
          break;
        case 'reflect':
          obj = ctx.serve;
          break;
        case 'scm':
          obj = ctx.repo;
          break;
        case 'plugin':
          obj = {
            'config': function* (opts) {}
          };
          break;
        case 'login':
          obj = {
            'getUserInfo': function* (opts) {
              return yield ctx.auth.user(opts || {});
            },
            'getUserTicket': function* (opts) {
              opts = opts || {};
              opts.app = opts.app || plugin.key;
              return yield ctx.auth.ticket(opts);
            }
          };
          break;
        default:
          usePlugin = true;
          break;
      }

      if (usePlugin) {
        return co.wrap(ctx.usePlugin.bind(ctx))(name)
          .then(function(p) {
            let fn = p.published[method];
            if (typeof fn != 'function') {
              return Promise.reject(new Error('not found method `' + method + '` in `' + p.key + '`.'));
            }
            return new Promise(function(resolve, reject) {
              let info = {'from': plugin.key};
              if (fn.length >= 3) { // as callback.
                return fn.call(null, args, info, function(err, data) {
                  if (err) return reject(err);
                  return resolve(data);
                });
              }

              let ret = fn.call(plugin, args, info);
              if (ret && ('function' == typeof ret.then)) {
                ret.then(resolve).catch(reject)
              } else {
                // let co rock.
                co(ret).then(resolve).catch(reject);
              }
            });
          });
      }

      if (typeof obj[method] != 'function') {
        return Promise.reject(new Error('not found method `' + method + '` in `' + name + '`.'));
      }

      return co.wrap(obj[method]).bind(obj)(args);
    };

    // depreted
    runtime.publish = function(name, fn) {
      ctx.log.verbose('[core]', 'p `%s` publish `%s`', plugin.key, name);
      plugin.published[name] = fn;
    };

    // depreted
    runtime.require = ctx.store.require.bind(ctx.store);
  }
};